# Generated by Django 5.1.6 on 2025-02-24 12:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='feed',
            name='freshness',
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='feed',
            name='feed_type',
            field=models.CharField(choices=[('rss', 'Rss'), ('atom', 'Atom'), ('skeleton', 'Skeleton'), ('search_index', 'Search Index')], editable=False, help_text='type of feed', max_length=12),
        ),
    ]
